SoundAlertSuite = {init={}}
SoundAlertSuite.events_index = {} --list of indexes corresponding to vars.alerts with EVENT_NAME as key
SoundAlertSuite.playtimes = {} --timestamps of when sound last played per alert
SoundAlertSuite.licenses = {} --player's current licenses used by license increase alert
SoundAlertSuite.version = '1.1.3'

--## Choco ##--

SoundAlertSuite.ui, SoundAlertSuite.vars = dofile('ui.lua')

-- INITIALIZATION --

--Register events corresponding to enabled items when entering game and call init function if defined.
local function init_events()
	--Unregister events for all alerts (for case of logging out and back in again)
	for event_name,_ in pairs(SoundAlertSuite.events_index) do
		UnregisterEvent(SoundAlertSuite, event_name)
	end
	SoundAlertSuite.events_index = {} --clear list
	
	for id,v in pairs(SoundAlertSuite.vars.alerts) do
		if v.enable then
			for _,event_name in ipairs(v.events) do
				if not SoundAlertSuite.events_index[event_name] then
					RegisterEvent(SoundAlertSuite, event_name) --event not registered yet
					SoundAlertSuite.events_index[event_name] = {} --table doesn't exist yet, create
				end
				
				table.insert(SoundAlertSuite.events_index[event_name], id) --add to list of alerts using this event
				if v.init then
					SoundAlertSuite[v.init]() --call init function
				end
			end
		end
	end
end

function SoundAlertSuite.init:PLAYER_ENTERED_GAME(event, data)
	RegisterEvent(SoundAlertSuite.init, "HUD_SHOW")
	RegisterEvent(SoundAlertSuite.init, "SHOW_STATION")
end

function SoundAlertSuite.init:HUD_SHOW(event, data) --Player logging in to space
	UnregisterEvent(SoundAlertSuite.init, "HUD_SHOW")
	UnregisterEvent(SoundAlertSuite.init, "SHOW_STATION")
	
	init_events()
end

function SoundAlertSuite.init:SHOW_STATION(event, data) --Player logging in to station
	UnregisterEvent(SoundAlertSuite.init, "HUD_SHOW")
	UnregisterEvent(SoundAlertSuite.init, "SHOW_STATION")
	
	init_events()
end

-- EVENT HANDLING --

function SoundAlertSuite:OnEvent(event, ...)
	local events = SoundAlertSuite.vars.alerts
	for _,id in ipairs(SoundAlertSuite.events_index[event]) do
		if events[id] and events[id].enable and ( not(events[id].func) or SoundAlertSuite[events[id].func](event, ...) ) then
			if SoundAlertSuite.vars.is_sound_alert then
				SoundAlertSuite.PlaySound(SoundAlertSuite.vars.sound_name[events[id].sound], events[id].vol or SoundAlertSuite.vars.vol_master, id)
			end
			
			if SoundAlertSuite.vars.is_flash_alert and events[id].flash_color~="0 0 0" then
				SoundAlertSuite.ActivateFlash(events[id].flash_color, events[id].title)
			end
		end
	end
end

--save record of player's current licenses
--enable==true or nil: save record, enable==false: clear record
function SoundAlertSuite.saveLicenses(enable)
	if enable ~= false then --exclude nil case
		--save current licenses
		local all_zero = true
		for n=1,5 do
			SoundAlertSuite.licenses[n] = GetLicenseLevel(n) --Doesn't work on first PLAYER_ENTERED_GAME --fixed
			if SoundAlertSuite.licenses[n] > 0 then all_zero = false end
		end 
		
		--try another approach if all licenses came back zero
		if all_zero then
			local char_name = GetPlayerName()
			for cslot=1,6 do
				local name, _, _, _, _, _, _, l1, l2, l3, l4, l5 = GetCharacterInfo(cslot)
				if name and name==char_name then
					SoundAlertSuite.licenses = {l1, l2, l3, l4, l5}
					break --don't bother checking other characters
				end
			end
		end
	else SoundAlertSuite.licenses = {} end --clear saved licenses
end

function SoundAlertSuite.isChatAlert(event, data)
	return data.channelid==tonumber(SoundAlertSuite.vars.alerts[1].data) and data.name~=GetPlayerName()
end
function SoundAlertSuite.isBuddyEntered(event, data)
	return data.msg:find("Your buddy .+ logged on%.")
end
function SoundAlertSuite.isBuddyLeft(event, data)
	return data.msg:find("Your buddy .+ logged off%.")
end
function SoundAlertSuite.isBuddyRequest(event, data)
	return data.msg:find(".+ requests permission to become your buddy%.") or data.msg:find(".+ is now your buddy%.")
end
function SoundAlertSuite.isReceivedCredits(event, data)
	return data.msg:find(".+ sent you %d+ credits%.")
end
function SoundAlertSuite.isScoredKill(event, ...) --return 1 for player kill, 2 for NPC, else false
	local victim, killer, weaponid = ...
	local isNPC = GetPlayerName(victim):match("^*")
	if killer==GetCharacterID() and victim~=GetCharacterID() then
		-- return SoundAlertSuite.vars.alerts[12].option_enable or not isNPC --OBSOLETE
		return (not isNPC) and 1 or (SoundAlertSuite.vars.alerts[12].option_enable and 2 or false)
	else return false end
end
function SoundAlertSuite.isLicenseIncreased(event, _)
	local is_increase = false
	for n=1,5 do
		local val = GetLicenseLevel(n)
		if SoundAlertSuite.licenses[n] and val > SoundAlertSuite.licenses[n] then --compare current license levels to saved levels
			SoundAlertSuite.licenses[n] = val
			is_increase = true
		end
	end
	return is_increase
end
function SoundAlertSuite.isOutOfAmmo(event, itemid)
	local ammo = GetAddonItemInfo(itemid)
	return ammo and ammo==0
end

-- END EVENT HANDLING --

--Plays specified sound by name at specified volume (0-1)
--id corresponds to the alert to ensure the sound is only played once every 200ms per alert
--returns false if playback failed due to incorrect parameters specified or less than 200ms since last
function SoundAlertSuite.PlaySound(sound_name, vol, id)
	local time = gkmisc.GetGameTime()
	local last_play = SoundAlertSuite.playtimes[id]
	
	sound_name = tostring(sound_name)
	vol = tonumber(vol)
	
	if not sound_name or not vol then return false end
	if vol < 0 or vol > 1 then return false end
	
	if not last_play or time > last_play + SoundAlertSuite.vars.min_snd_replay then
		gksound.GKPlaySound(sound_name, vol)
		SoundAlertSuite.playtimes[id] = time
		
		return true
	else return false end
end

--Flash screen specified color ("255 255 255") with str description to display on hud
function SoundAlertSuite.ActivateFlash(color, str)
	str = tostring(str)
	
	if str then
		HUD.hitby_text.title = str
		HUD.hitby_text.fgcolor = color
		FadeControl(HUD.hitby_text, 2, 4, 0)
	end
		
	HUD.blood_flash.bgcolor = color.." 0 +"
	HUD.blood_flash.visible = "YES"
	FadeControl(HUD.blood_flash, 0.5, FlashIntensity, 0) --use flash intensitiy specified in game options
	
	return true
end

--Sound test
function SoundAlertSuite.test(id)
	local events = SoundAlertSuite.vars.alerts
	if events[id] then gksound.GKPlaySound(vars.sound_name[events[id].sound], events[id].vol or SoundAlertSuite.var.vol_master) end
end

-- HANDLE COMMAND LINE INPUT --

--Input from console
SoundAlertSuite.Inputs = {sound=true,flash=true}
function SoundAlertSuite:Input(args)
	args = args or {}
	local command = args[1] and args[1]:lower()
	if command then table.remove(args, 1) end
	
	if SoundAlertSuite.Inputs[command] then
		SoundAlertSuite[command](SoundAlertSuite,args)
	else SoundAlertSuite.ui:Show() end
end

--if successful returns saved val, nil if failed
local function set_vars(self, key, args)
	args = args or {}
	local param = args[1] and args[1]:lower()
	local inputs = {on=true,off=false,toggle=not self.vars['is_'..key],yes=true,no=false,['true']=true,['false']=false} --keys are valid entries for param
	
	if inputs[param]~=nil then
		return self.vars.WriteVal(key, inputs[param])
	else return end
end

function SoundAlertSuite:sound(args)
	local val = set_vars(self, 'sound_alert', args)
	if val~=nil then
		print('\127ffffffSoundAlertSuite sound playback is '..(val and 'ON' or 'OFF'))
		if self.ui.dialog and self.ui.dialog.visible=="YES" then self.ui.Update(self.ui) end --update ui if visible
	else print'\127ffffff/sas sound \12700ff00on\127o/\12700ff00off\127o/\12700ff00toggle\127o - set sound playback' end --invalid parameters
end

function SoundAlertSuite:flash(args)
	local val = set_vars(self, 'flash_alert', args)
	if val~=nil then
		print('\127ffffffSoundAlertSuite flash alerts are '..(val and 'ON' or 'OFF'))
		if self.ui.dialog and self.ui.dialog.visible=="YES" then self.ui.Update(self.ui) end --update ui if visible
	else print'\127ffffff/sas flash \12700ff00on\127o/\12700ff00off\127o/\12700ff00toggle\127o - set flash alerts' end --invalid parameters
end

--function SoundAlertSuite:help() SoundAlertSuite.ui:ShowHelp() end --DELETE? displaying help dialog directly might be too confusing

-- END COMMAND LINE INPUT --

RegisterUserCommand("SAS", SoundAlertSuite.Input)
RegisterUserCommand("SoundAlertSuite", SoundAlertSuite.Input)
RegisterEvent(SoundAlertSuite.init, "PLAYER_ENTERED_GAME")
